package com.biganto.visual.roompark.domain.model

import androidx.annotation.StringRes
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.data.repository.db.requrey.model.DealEntity
import com.biganto.visual.roompark.data.repository.db.requrey.model.EstateEntity
import com.biganto.visual.roompark.data.repository.db.requrey.model.ExplicationEntity
import com.biganto.visual.roompark.data.repository.db.requrey.model.PlanPreset

/**
 * Created by Vladislav Bogdashkin on 23.09.2019.
 */

const val DEFAULT_WALLS_FEATURE_NAME = "walls"

data class DealListModel(val deals:List<DealListModel>)


data class StatusModel(
    val orderId:Int,
    val title:String,
    val shortTitle:String
)

data class DealModel(
    val id:String,
    val estate_id:String,
    val opportunitySum:Int,
    val paymentSum:Int,
    val amount_pay_sum:Int,
    val statusId:Int,
    val managerName: String,
    val estate:EstateModel
)

fun fromEntity(entity:DealEntity) = DealModel(
    id = entity.id,
    estate_id = entity.estateCrmId,
    opportunitySum = entity.opportunitySum,
    paymentSum = entity.paymentSum,
    amount_pay_sum = entity.amountPaySum,
    statusId = entity.statusId,
    managerName = entity.managerName,
    estate = fromEntity(entity.estate as EstateEntity)
)


data class EstateModel(
    val id:Int,
    val type:FlatType,
    val number:String,
    val sectionBegin:Int?=null,
    val sectionEnd:Int?=null,
    val planPNG:PlanModel?,
    val planJPG:PlanModel?,
    val rooms:Int?=null,
    val albumId:Int?=null,
    val multitourId:Int?=null,
    val multitourPreview:String?=null,
    val commonInfo:CommonInfoModel? = null,
    val url:String?
)

enum class FlatType{
    FLAT,
    PARKING,
    STORAGE
}

@StringRes fun FlatType.typeDoubleString() =
    when(this){
        FlatType.FLAT -> R.string.estate_type_flat_long
        FlatType.PARKING -> R.string.estate_type_parking_long
        FlatType.STORAGE -> R.string.estate_type_store_long
        else -> R.string.estate_type_other_long
    }

@StringRes fun FlatType.typeShortString() =
    when(this){
        FlatType.FLAT -> R.string.estate_type_flat_short
        FlatType.PARKING -> R.string.estate_type_parking_short
        FlatType.STORAGE -> R.string.estate_type_store_short
        else -> R.string.estate_type_other_short
    }

fun fromEntity(entity:EstateEntity): EstateModel {
    val model = EstateModel(
        id = entity.id,
    type = FlatType.valueOf(entity.type.toUpperCase()),
    number = entity.number,
    sectionBegin = entity.sectionBegin,
    sectionEnd = entity.sectionEnd,
    planPNG = null,
    planJPG = null,
    rooms = entity.rooms,
    albumId = entity.albumId,
    multitourId = entity.multitourId,
    multitourPreview = entity.multitourPreview,
        commonInfo = CommonInfoModel(
            building = entity.info_building,
            section_begin = entity.sectionBegin,
            floor = entity.info_floor,
            floor_max = entity.info_floor_max,
            area = entity.info_area,
            area_living = entity.info_area_living,
            kind = entity.info_kind,
            dependent = entity.info_dependent,
            decoration = entity.info_decoration,
            ceiling = entity.info_ceiling,
            windows_face = entity.info_window_face,
            direction = entity.info_direction,
            price_meter = entity.info_price_meter,
            price = entity.info_price,
//            discount = entity.di
//            discount_amount = entity
            rooms = entity.rooms
        ),
        url = entity.url
    )
    return model
}

data class PlanModel(
    val url:String,
    val width:Int,
    val height:Int
)


data class CommonInfoModel(
    val building: Int,
    val section_begin: Int?,
    val floor: Int,
    val floor_max: Int,
    val area: Float?,
    val area_living: Float?,
    val kind: String?,
    val dependent:Boolean?=null,
    val decoration: String?,
    val ceiling: Float?,
    val windows_face: String?,
    val direction: String?,
    val price_meter: Int?,
    val price: Int?,
    val discount: Float? = null,
    val discount_amount: Int? = null,
    val rooms: Int?
)

fun fromEntity(entity : PlanPreset) = PlanPresetModel(
        planId = entity.planId,
        estateId = entity.estateId.id,
        title = entity.title,
        features = entity.features.asSequence()
            .map { FeatureModel.fromFeature(it) }.toList(),
            //default true value for walls (otherwise plan may be empty)
        explication = List(entity.explication.size)
        { ExplicationModel((entity.explication[it] as ExplicationEntity))}
    )

data class PlanPresetModel(
    val planId:Int,
    val explication:List<ExplicationModel>,
    var estateId:Int,
    val title:String,
    val features:List<FeatureModel>
){

    inline fun <reified T : FeatureModel> switchFeature(value:Boolean){
        features.first { it is T}.switchedOn = value
    }
}

data class ExplicationModel(
    val living: Boolean,
    val area: Float,
    val title: String
){
    constructor(entity:ExplicationEntity):this(entity.living,entity.area,entity.title)
}



/*
data class CommonEstateInfoModel(
    val corpus:Int,
    val section:Int,
    val floor:Int,
    val totalFloors:Int,
    val totalArea:Float,
    val livingArea:Float,
    val rooms:Int,
    val flatType:String,
    val furnishType:String,
    val ceilingHieght:Float,
    val windowViewTGype:String,
    val sideOfTheWorld:String,
    val costPerM2:Float,
    val totalSum:Float
)
*/
